@@ -1,5 +1,5 @@
 // elaboration functions
-// Copyright (C) 2005 Red Hat Inc.
+// Copyright (C) 2005, 2006 Red Hat Inc.
 //
 // This file is part of systemtap, and is free software.  You can
 // redistribute it and/or modify it under the terms of the GNU General
@@ -54,7 +54,8 @@ derived_probe::derived_probe (probe *p):
 derived_probe::derived_probe (probe *p, probe_point *l):
   base (p)
 {
-  this->locations.push_back (l);
+  if (l)
+    this->locations.push_back (l);
   if (p)
     {
       this->tok = p->tok;
@@ -391,10 +392,10 @@ systemtap_session::register_library_aliases()
             }
           catch (const semantic_error& e)
             {
-              print_error (e);
-              cerr << "         while: registering probe alias ";
+              cerr << "while: registering probe alias ";
               alias->printsig(cerr);
               cerr << endl;
+              print_error (e);
             }
 	}
     }
@@ -442,7 +443,7 @@ derive_probes (systemtap_session& s,
         {
           // XXX: prefer not to print_error at every nest/unroll level
           s.print_error (e);
-          cerr << "         while: resolving probe point " << *loc << endl;
+          cerr << "while: resolving probe point " << *loc << endl;
           //          if (! exc_outermost)
           //            throw;
         }
@@ -935,7 +936,11 @@ symresolution_info::visit_foreach_loop (foreach_loop* e)
 	  if (d)
 	    array->referent = d;
 	  else
-	    throw semantic_error ("unresolved global array " + array->name, e->tok);
+            {
+              cerr << "while searching for arity " << e->indexes.size()
+                   << " array:\n";
+              throw semantic_error ("unresolved global array " + array->name, e->tok);
+            }
 	}
     }
   else 
@@ -1071,7 +1076,10 @@ symresolution_info::visit_functioncall (functioncall* e)
   if (d)
     e->referent = d;
   else
-    throw semantic_error ("unresolved function call", e->tok);
+    {
+      cerr << "while searching for arity " << e->args.size() << " function:\n";
+      throw semantic_error ("unresolved function call", e->tok);
+    }
 }
 
 
@@ -2268,6 +2276,7 @@ typeresolution_info::visit_print_format (print_format* e)
 	    case print_format::conv_signed_decimal:
 	    case print_format::conv_unsigned_decimal:
 	    case print_format::conv_unsigned_octal:
+	    case print_format::conv_unsigned_ptr:
 	    case print_format::conv_unsigned_uppercase_hex:
 	    case print_format::conv_unsigned_lowercase_hex:
 	      wanted = pe_long;
