@@ -1,6 +1,7 @@
 // systemtap translator/driver
-// Copyright (C) 2005 Red Hat Inc.
+// Copyright (C) 2005-2006 Red Hat Inc.
 // Copyright (C) 2005 IBM Corp.
+// Copyright (C) 2006 Intel Corporation.
 //
 // This file is part of systemtap, and is free software.  You can
 // redistribute it and/or modify it under the terms of the GNU General
@@ -27,6 +28,7 @@ extern "C" {
 #include <sys/times.h>
 #include <sys/time.h>
 #include <time.h>
+#include <elfutils/libdwfl.h>
 }
 
 using namespace std;
@@ -38,9 +40,9 @@ version ()
   clog
     << "SystemTap translator/driver "
     << "(version " << VERSION << " built " << DATE << ")" << endl
+    << "(Using " << dwfl_version (NULL) << " libraries.)" << endl
     << "Copyright (C) 2005-2006 Red Hat, Inc. and others" << endl
-    << "This is free software; see the source for copying conditions."
-    << endl;
+    << "This is free software; see the source for copying conditions." << endl;
 }
 
 void
@@ -67,7 +69,7 @@ usage (systemtap_session& s)
     << "   -b         bulk (relayfs) mode" << (s.bulk_mode ? " [set]" : "") << endl
     << "   -s NUM     buffer size in megabytes, instead of "
     << s.buffer_size << endl
-    << "   -p NUM     stop after pass NUM 1-5, instead of " 
+    << "   -p NUM     stop after pass NUM 1-5, instead of "
     << s.last_pass << endl
     << "              (parse, elaborate, translate, compile, run)" << endl
     << "   -I DIR     look in DIR for additional .stp script files";
@@ -86,7 +88,7 @@ usage (systemtap_session& s)
     << "   -m MODULE  set probe module name, instead of "
     << s.module_name << endl
     << "   -o FILE    send output to file, instead of stdout" << endl
-    << "   -c CMD     start the probes, run CMD, and exit when it finishes" 
+    << "   -c CMD     start the probes, run CMD, and exit when it finishes"
     << endl
     << "   -x PID     sets target() to PID" << endl
     ;
@@ -257,7 +259,7 @@ main (int argc, char * const argv [])
   int rc = 0;
 
   // override PATH and LC_ALL
-  char* path = "PATH=/bin:/sbin:/usr/bin:/usr/sbin"; 
+  char* path = "PATH=/bin:/sbin:/usr/bin:/usr/sbin";
   char* lc_all = "LC_ALL=C";
   rc = putenv (path) || putenv (lc_all);
   if (rc)
@@ -266,7 +268,7 @@ main (int argc, char * const argv [])
       cerr << "setenv (\"" << path << "\" + \"" << lc_all << "\"): "
            << e << endl;
     }
-  
+
 
   // arguments parsed; get down to business
 
@@ -312,23 +314,26 @@ main (int argc, char * const argv [])
 
   // Construct arch / kernel-versioning search path
   vector<string> version_suffixes;
-  const string& kvr = s.kernel_release;
+  string kvr = s.kernel_release;
   const string& arch = s.architecture;
   // add full kernel-version-release (2.6.NN-FOOBAR) + arch
   version_suffixes.push_back ("/" + kvr + "/" + arch);
   version_suffixes.push_back ("/" + kvr);
   // add kernel version (2.6.NN) + arch
-  string::size_type dash_rindex = kvr.rfind ('-');
-  if (dash_rindex > 0 && dash_rindex != string::npos) {
-    version_suffixes.push_back ("/" + kvr.substr (0, dash_rindex) + "/" + arch);
-    version_suffixes.push_back ("/" + kvr.substr (0, dash_rindex));
+  string::size_type dash_index = kvr.find ('-');
+  if (dash_index > 0 && dash_index != string::npos) {
+    kvr.erase(dash_index);
+    version_suffixes.push_back ("/" + kvr + "/" + arch);
+    version_suffixes.push_back ("/" + kvr);
   }
   // add kernel family (2.6) + arch
-  string::size_type dot_index = kvr.find ('.');
-  string::size_type dot2_index = kvr.find ('.', dot_index+1);
-  if (dot2_index > 0 && dot2_index != string::npos) {
-    version_suffixes.push_back ("/" + kvr.substr (0, dot2_index) + "/" + arch);
-    version_suffixes.push_back ("/" + kvr.substr (0, dot2_index));
+  string::size_type dot1_index = kvr.find ('.');
+  string::size_type dot2_index = kvr.rfind ('.');
+  while (dot2_index > dot1_index && dot2_index != string::npos) {
+    kvr.erase(dot2_index);
+    version_suffixes.push_back ("/" + kvr + "/" + arch);
+    version_suffixes.push_back ("/" + kvr);
+    dot2_index = kvr.rfind ('.');
   }
   // add architecture search path
   version_suffixes.push_back("/" + arch);
@@ -405,7 +410,7 @@ main (int argc, char * const argv [])
 
   if (rc)
     cerr << "Pass 1: parse failed.  "
-         << (s.verbose ? "" : "Try again with '-v' (verbose) option.")
+         << "Try again with more '-v' (verbose) options."
          << endl;
 
   if (rc || s.last_pass == 1) goto cleanup;
@@ -486,7 +491,7 @@ main (int argc, char * const argv [])
 
   if (rc)
     cerr << "Pass 2: analysis failed.  "
-         << (s.verbose ? "" : "Try again with '-v' (verbose) option.")
+         << "Try again with more '-v' (verbose) options."
          << endl;
 
   if (rc || s.last_pass == 2) goto cleanup;
@@ -516,7 +521,7 @@ main (int argc, char * const argv [])
 
   if (rc)
     cerr << "Pass 3: translation failed.  "
-         << (s.verbose ? "" : "Try again with '-v' (verbose) option.")
+         << "Try again with more '-v' (verbose) options."
          << endl;
 
   if (rc || s.last_pass == 3) goto cleanup;
@@ -536,15 +541,19 @@ main (int argc, char * const argv [])
 
   if (rc)
     cerr << "Pass 4: compilation failed.  "
-         << (s.verbose ? "" : "Try again with '-v' (verbose) option.")
+         << "Try again with more '-v' (verbose) options."
          << endl;
 
-  // XXX: what to do if rc==0 && last_pass == 4?  dump .ko file to stdout? 
+  // XXX: what to do if rc==0 && last_pass == 4?  dump .ko file to stdout?
   if (rc || s.last_pass == 4) goto cleanup;
 
   // PASS 5: RUN
   times (& tms_before);
   gettimeofday (&tv_before, NULL);
+  // NB: this message is a judgement call.  The other passes don't emit
+  // a "hello, I'm starting" message, but then the others aren't interactive
+  // and don't take an indefinite amount of time.
+  if (s.verbose) clog << "Pass 5: starting run." << endl;
   rc = run_pass (s);
   times (& tms_after);
   gettimeofday (&tv_after, NULL);
@@ -554,7 +563,7 @@ main (int argc, char * const argv [])
 
   if (rc)
     cerr << "Pass 5: run failed.  "
-         << (s.verbose ? "" : "Try again with '-v' (verbose) option.")
+         << "Try again with more '-v' (verbose) options."
          << endl;
 
   // if (rc) goto cleanup;
