@@ -1,7 +1,7 @@
 /* -*- linux-c -*-
  *
  * /proc transport and control
- * Copyright (C) 2005 Red Hat Inc.
+ * Copyright (C) 2005, 2006 Red Hat Inc.
  *
  * This file is part of systemtap, and is free software.  You can
  * redistribute it and/or modify it under the terms of the GNU General
@@ -9,7 +9,7 @@
  * later version.
  */
 
-#define STP_DEFAULT_BUFFERS 128
+#define STP_DEFAULT_BUFFERS 256
 static int _stp_current_buffers = STP_DEFAULT_BUFFERS;
 
 static struct list_head _stp_ready_q;
@@ -31,13 +31,13 @@ _stp_proc_read (struct file *file, char __user *buf, size_t count, loff_t *ppos)
 		return -EINVAL;
 
 	out.cpu = cpu;
-#ifdef RELAYFS_VERSION_GE_4
+#if (RELAYFS_CHANNEL_VERSION >= 4) || defined (CONFIG_RELAY)
 	out.produced = _stp_chan->buf[cpu]->subbufs_produced;
 	out.consumed = _stp_chan->buf[cpu]->subbufs_consumed;
 #else
 	out.produced = atomic_read(&_stp_chan->buf[cpu]->subbufs_produced);
 	out.consumed = atomic_read(&_stp_chan->buf[cpu]->subbufs_consumed);
-#endif /* RELAYFS_VERSION_GE_4 */
+#endif  /* RELAYFS_CHANNEL_VERSION >= 4 || CONFIG_RELAY */
 
 	num = sizeof(out);
 	if (copy_to_user(buf, &out, num))
@@ -103,7 +103,8 @@ static ssize_t _stp_proc_write_cmd (struct file *file, const char __user *buf,
 			return 0;
 		if (copy_from_user (&ti, &buf[4], sizeof(struct transport_info)))
 			return -EFAULT;
-		_stp_transport_open (&ti);
+		if (_stp_transport_open (&ti) < 0)
+			return -1;
 		break;
 	}
 	default:
@@ -132,7 +133,9 @@ static int _stp_write (int type, void *data, int len)
 	spin_lock(&_stp_ready_lock);
 	if (!list_empty(&_stp_ready_q)) {
 		bptr = (struct _stp_buffer *)_stp_ready_q.prev;
-		if (bptr->len + len <= STP_BUFFER_SIZE && bptr->type == type) {
+		if (bptr->len + len <= STP_BUFFER_SIZE 
+		    && type == STP_REALTIME_DATA 
+		    && bptr->type == STP_REALTIME_DATA) {
 			memcpy (bptr->buf + bptr->len - 1, data, len);
 			bptr->len += len - 1;
 			spin_unlock(&_stp_ready_lock);
@@ -170,22 +173,23 @@ _stp_proc_read_cmd (struct file *file, char __user *buf, size_t count, loff_t *p
 {
 	struct _stp_buffer *bptr;
 	int len;
+	unsigned long flags;
 
 	/* wait for nonempty ready queue */
-	spin_lock(&_stp_ready_lock);
+	spin_lock_irqsave(&_stp_ready_lock, flags);
 	while (list_empty(&_stp_ready_q)) {
-		spin_unlock(&_stp_ready_lock);
+		spin_unlock_irqrestore(&_stp_ready_lock, flags);
 		if (file->f_flags & O_NONBLOCK)
 			return -EAGAIN;
 		if (wait_event_interruptible(_stp_proc_wq, !list_empty(&_stp_ready_q)))
 			return -ERESTARTSYS;
-		spin_lock(&_stp_ready_lock);
+		spin_lock_irqsave(&_stp_ready_lock, flags);
 	}
   
 	/* get the next buffer off the ready list */
 	bptr = (struct _stp_buffer *)_stp_ready_q.next;
 	list_del_init(&bptr->list);
-	spin_unlock(&_stp_ready_lock);
+	spin_unlock_irqrestore(&_stp_ready_lock, flags);
 
 	/* write it out */
 	len = bptr->len + 4;
@@ -198,9 +202,9 @@ _stp_proc_read_cmd (struct file *file, char __user *buf, size_t count, loff_t *p
 	}
 
 	/* put it on the pool of free buffers */
-	spin_lock(&_stp_pool_lock);
+	spin_lock_irqsave(&_stp_pool_lock, flags);
 	list_add_tail(&bptr->list, &_stp_pool_q);
-	spin_unlock(&_stp_pool_lock);
+	spin_unlock_irqrestore(&_stp_pool_lock, flags);
 
 	return len;
 }
@@ -228,13 +232,13 @@ static int _stp_set_buffers(int num)
 {
 	int i;
 	struct list_head *p;
-	
+	unsigned long flags;
+
 	//printk("stp_set_buffers %d\n", num);
 
 	if (num == 0 || num == _stp_current_buffers)
 		return _stp_current_buffers;
 	
-	spin_lock(&_stp_pool_lock);
 	if (num > _stp_current_buffers) {
 		for (i = 0; i < num - _stp_current_buffers; i++) {
 			p = (struct list_head *)kmalloc(sizeof(struct _stp_buffer),GFP_KERNEL);
@@ -242,21 +246,56 @@ static int _stp_set_buffers(int num)
 				_stp_current_buffers += i;
 				goto err;
 			}
+			spin_lock_irqsave(&_stp_pool_lock, flags);
 			list_add (p, &_stp_pool_q);
+			spin_unlock_irqrestore(&_stp_pool_lock, flags);
 		}
 	} else {
 		for (i = 0; i < _stp_current_buffers - num; i++) {
+			spin_lock_irqsave(&_stp_pool_lock, flags);
 			p = _stp_pool_q.next;
 			list_del(p);
+			spin_unlock_irqrestore(&_stp_pool_lock, flags);
 			kfree(p);
 		}
 	}
 	_stp_current_buffers = num;
 err:
-	spin_unlock(&_stp_pool_lock);
 	return _stp_current_buffers;
 }
 
+#if defined (STP_RELAYFS) && defined(CONFIG_RELAY)
+struct dentry *module_dir_dentry;
+
+static inline struct dentry *_stp_get_proc_root(void)
+{
+	struct file_system_type *procfs_type;
+	struct super_block *procfs_sb;
+
+	procfs_type = get_fs_type("proc");
+	if (!procfs_type || list_empty(&procfs_type->fs_supers))
+		return NULL;
+	procfs_sb = list_entry(procfs_type->fs_supers.next,
+			       struct super_block, s_instances);
+	return procfs_sb->s_root;
+}
+
+static inline struct dentry *_stp_force_dir_creation(const char *dirname, struct dentry *parent)
+{
+	struct dentry *dir_dentry;
+
+	mutex_lock(&parent->d_inode->i_mutex);
+	dir_dentry = lookup_one_len(dirname, parent, strlen(dirname));
+	mutex_unlock(&parent->d_inode->i_mutex);
+	if (IS_ERR(dir_dentry)) {
+		dir_dentry = NULL;
+		remove_proc_entry(dirname, NULL);
+	}
+	
+	return dir_dentry;
+}
+#endif  /* STP_RELAYFS && CONFIG_RELAY */
+
 static int _stp_register_procfs (void)
 {
 	int i;
@@ -265,6 +304,10 @@ static int _stp_register_procfs (void)
 	int j;
 	char buf[8];
 #endif
+#if defined (CONFIG_RELAY)
+	struct dentry *proc_root_dentry;
+	struct dentry *systemtap_dir_dentry;
+#endif /* CONFIG_RELAY */
 	struct proc_dir_entry *de;
 	struct list_head *p, *tmp;
 
@@ -296,11 +339,23 @@ static int _stp_register_procfs (void)
 			goto err0;
 	}
 
+#if defined (STP_RELAYFS) && defined (CONFIG_RELAY)
+	proc_root_dentry = _stp_get_proc_root();
+	systemtap_dir_dentry = _stp_force_dir_creation(dirname, proc_root_dentry);
+	if (!systemtap_dir_dentry)
+		goto err0;
+#endif /* STP_RELAYFS && CONFIG_RELAY */
 	/* now create /proc/systemtap/module_name */
 	_stp_proc_mod = proc_mkdir (THIS_MODULE->name, _stp_proc_root);
 	if (_stp_proc_mod == NULL)
 		goto err0;
 
+#if defined (STP_RELAYFS) && defined (CONFIG_RELAY)
+	module_dir_dentry = _stp_force_dir_creation(THIS_MODULE->name, systemtap_dir_dentry);
+	if (!module_dir_dentry)
+		goto err0;
+#endif /* STP_RELAYFS && CONFIG_RELAY */
+
 #ifdef STP_RELAYFS	
 	/* now for each cpu "n", create /proc/systemtap/module_name/n  */
 	for_each_cpu(i) {
