@@ -401,6 +401,10 @@ print_format::components_to_string(vector<format_component> const & components)
 	      oss << "llo";
 	      break;
 
+	    case conv_unsigned_ptr:
+	      oss << "p";
+	      break;
+
 	    case conv_unsigned_uppercase_hex:
 	      oss << "llX";
 	      break;
@@ -448,7 +452,7 @@ print_format::string_to_components(string const & str)
 	  assert (curr.type == conv_unspecified || curr.type == conv_literal);
 	  curr.type = conv_literal;
 	  curr.literal_string += '%';
-	  ++i;
+          i += 2;
 	  continue;
 	}
       else 
@@ -551,6 +555,10 @@ print_format::string_to_components(string const & str)
 	  curr.type = conv_unsigned_decimal;
 	  break;
 	  
+	case 'p':
+	  curr.type = conv_unsigned_ptr;
+	  break;
+
 	case 'X':
 	  curr.type = conv_unsigned_uppercase_hex;
 	  break;
@@ -1472,7 +1480,7 @@ void
 varuse_collecting_visitor::visit_embeddedcode (embeddedcode *s)
 {
   // In order to elide unused but correct functions generated to
-  // get/set $target variables, we encode our knowledge that such
+  // get $target variables, we encode our knowledge that such
   // functions are side-effect-free.  We tell them apart from ordinary
   // tapset embedded-C functions by the naming prefix.  XXX Something
   // apart from this heuristic would be nice.  XXX Similarly, some
@@ -1481,8 +1489,9 @@ varuse_collecting_visitor::visit_embeddedcode (embeddedcode *s)
   
   assert (current_function); // only they get embedded code
   string name = current_function->name;
-  if (name.length() > 6 && name.substr(0, 6) == "_tvar_")
+  if (name.length() > 10 && name.substr(0, 10) == "_tvar_get_")
     return;
+  // NB: setter functions naturally have side-effects
 
   embedded_seen = true;
 }
@@ -1541,6 +1550,7 @@ varuse_collecting_visitor::visit_symbol (symbol *e)
 // NB: stat_op need not be overridden, since it will get to
 // visit_symbol and only as a possible rvalue.
 
+
 void
 varuse_collecting_visitor::visit_arrayindex (arrayindex *e)
 {
